from typing import List, Union

from docarray import BaseDoc, DocList
from docarray.typing import NdArray
from pydantic import Field, BaseModel

from api_schemas.base import BaseInputModel


class ExecutorUsage(BaseDoc):
    """Usage model containing the usage of the embedding services to report"""
    total_tokens: int = Field(description='The number of tokens used to embed the input text')


# EXECUTOR MODELS
## Model to be imported by the Executor and used by the Universal API to communicate with it
class TextDoc(BaseDoc):
    """A Document containing text field"""
    text: str


## Model to be imported by the Executor and used by the Universal API to communicate with it
class EmbeddingDoc(BaseDoc):
    """A Document to be returned by the embedding backend with the result of the embedding and the usage for that
    individual text """
    embedding: NdArray = Field(description='The embedding of the texts', default=[])
    usage: ExecutorUsage


# UNIVERSAL API MODELS (mimic OpenAI API)
class TextEmbeddingInput(BaseInputModel):
    """The input to the Universal API for text embedding. OpenAI compatible """
    inputs: Union[List[str], str] = Field(
        description='List of texts to embed. The length of the list must be between `1` and `100`',
        min_items=1,
        max_items=100,
    )

    class Config(BaseDoc.Config):
        allow_population_by_field_name = True
        arbitrary_types_allowed = True
        schema_extra = {
            "example": {
                "model": "jina-embedding-s-en-v1",
                "inputs": ["Hello, world!"],
            },
        }


class EmbeddingObject(BaseModel):
    """Embedding object. OpenAI compatible"""
    object: str = 'embedding'
    index: int = Field(description='The index of the embedding in the list of inputs')
    embedding: List[float] = Field(description='The embedding of the text', default=[])

    class Config(BaseDoc.Config):
        allow_population_by_field_name = True
        arbitrary_types_allowed = True
        schema_extra = {
            "example": {
                "embeddings": [[0.1, 0.2, 0.3]],
            }
        }


class Usage(BaseModel):
    total_tokens: int = Field(description='The number of tokens used all the texts in the input')
    prompt_tokens: int = Field(description='Same as total_tokens')


class ModelEmbeddingOutput(BaseInputModel):
    """Output of the embedding service"""
    object: str = 'list'
    data: List[EmbeddingObject] = Field(description='The list of Embedding Object resulting from the embedding service')
    usage: Usage = Field(description='Total usage of the request. Adding up all the usage of each individual input')
